@@ -10,18 +10,18 @@ module Ast where
 import GHC.Generics (Generic)
 
 newtype Program = Program Expression -- Q -> expr
-  deriving (Eq, Show)
+  deriving (Eq, Ord, Show)
 
 data Expression
-  = ExFormation [Binding] -- [bindings]
-  | ExThis
+  = ExFormation [Binding] -- [[ bindings ]]
+  | ExThis -- $
   | ExGlobal -- Q
   | ExTermination -- T
   | ExMeta String -- !e
   | ExApplication Expression Binding -- expr(attr -> expr)
   | ExDispatch Expression Attribute -- expr.attr
   | ExMetaTail Expression String -- expr * !t
-  deriving (Eq, Show, Generic)
+  deriving (Eq, Ord, Show, Generic)
 
 data Binding
   = BiTau Attribute Expression -- attr -> expr
@@ -30,14 +30,14 @@ data Binding
   | BiVoid Attribute -- attr ↦ ?
   | BiLambda String -- λ ⤍ Function
   | BiMetaLambda String -- λ ⤍ !F
-  deriving (Eq, Show, Generic)
+  deriving (Eq, Ord, Show, Generic)
 
 data Bytes
   = BtEmpty -- --
   | BtOne String -- 1F-
   | BtMany [String] -- 00-01-02-...04
   | BtMeta String -- !b
-  deriving (Eq, Show, Generic)
+  deriving (Eq, Ord, Show, Generic)
 
 data Attribute
   = AtLabel String -- attr
@@ -47,7 +47,16 @@ data Attribute
   | AtLambda -- λ, used only in yaml conditions
   | AtDelta -- Δ, used only in yaml conditions
   | AtMeta String -- !a
-  deriving (Eq, Show, Generic)
+  deriving (Eq, Generic, Ord)
+
+instance Show Attribute where
+  show (AtLabel label) = label
+  show (AtAlpha idx) = 'α' : show idx
+  show AtRho = "ρ"
+  show AtPhi = "φ"
+  show AtDelta = "Δ"
+  show AtLambda = "λ"
+  show (AtMeta meta) = '!' : meta
 
 countNodes :: Program -> Integer
 countNodes (Program expr) = countNodes' expr
