@@ -10,10 +10,11 @@ import Ast
 import Builder (contextualize)
 import Control.Exception (throwIO)
 import Data.List (partition)
+import Deps (BuildTermFunc, SaveStepFunc)
 import Misc
+import Must (Must (..))
 import Rewriter (RewriteContext (RewriteContext), rewrite')
 import Rule (RuleContext (RuleContext), isNF)
-import Term (BuildTermFunc)
 import Text.Printf (printf)
 import XMIR (XmirContext (XmirContext))
 import Yaml (normalizationRules)
@@ -21,11 +22,22 @@ import Yaml (normalizationRules)
 data DataizeContext = DataizeContext
   { _program :: Program,
     _maxDepth :: Integer,
-    _buildTerm :: BuildTermFunc
+    _maxCycles :: Integer,
+    _depthSensitive :: Bool,
+    _buildTerm :: BuildTermFunc,
+    _saveStep :: SaveStepFunc
   }
 
 switchContext :: DataizeContext -> RewriteContext
-switchContext DataizeContext {..} = RewriteContext _program _maxDepth _buildTerm 0
+switchContext DataizeContext {..} =
+  RewriteContext
+    _program
+    _maxDepth
+    _maxCycles
+    _depthSensitive
+    _buildTerm
+    MtDisabled
+    _saveStep
 
 maybeBinding :: (Binding -> Bool) -> [Binding] -> (Maybe Binding, [Binding])
 maybeBinding _ [] = (Nothing, [])
@@ -153,9 +165,6 @@ dataize' expr prog = do
     Just morphed -> dataize' morphed prog
     _ -> pure Nothing
 
-toDouble :: Integer -> Double
-toDouble = fromIntegral
-
 atom :: String -> Expression -> DataizeContext -> IO (Maybe Expression)
 atom "L_org_eolang_number_plus" self ctx = do
   left <- dataize' (ExDispatch self (AtLabel "x")) ctx
@@ -188,4 +197,4 @@ atom "L_org_eolang_number_eq" self ctx = do
         then pure (Just (DataNumber (numToBts first)))
         else pure (Just (ExDispatch self (AtLabel "y")))
     _ -> pure Nothing
-atom func _ _ = throwIO (userError (printf "Atom '%s' does not exist" func))
\ No newline at end of file
+atom func _ _ = throwIO (userError (printf "Atom '%s' does not exist" func))
