@@ -11,8 +11,10 @@ import Control.Exception (throwIO)
 import Control.Monad (replicateM, when)
 import qualified Data.ByteString.Char8 as B
 import Data.Char (intToDigit)
+import Data.Functor
 import Data.Set (Set)
 import qualified Data.Set
+import qualified Data.Set as Set
 import GHC.IO (unsafePerformIO)
 import Matcher
 import Misc
@@ -22,11 +24,11 @@ import Pretty
 import Random (randomString)
 import Regexp
 import System.Random (randomRIO)
-import Term
+import Deps
 import Text.Printf (printf)
 import qualified Yaml as Y
 
-buildTerm :: String -> [Y.ExtraArgument] -> Subst -> Program -> IO Term
+buildTerm :: BuildTermFunc
 buildTerm "contextualize" = _contextualize
 buildTerm "scope" = _scope
 buildTerm "random-tau" = _randomTau
@@ -38,31 +40,37 @@ buildTerm "size" = _size
 buildTerm "tau" = _tau
 buildTerm "string" = _string
 buildTerm "number" = _number
-buildTerm func  = _unsupported func
+buildTerm "sum" = _sum
+buildTerm "join" = _join
+buildTerm func = _unsupported func
 
-argToStrBytes :: Y.ExtraArgument -> Subst -> Program -> IO String
-argToStrBytes (Y.ArgBytes bytes) subst _ = do
-  bts <- buildBytesThrows bytes subst
-  pure (btsToUnescapedStr bts)
-argToStrBytes (Y.ArgExpression expr) subst prog = do
+argToBytes :: Y.ExtraArgument -> Subst -> Program -> IO Bytes
+argToBytes (Y.ArgBytes bytes) subst _ = buildBytesThrows bytes subst
+argToBytes (Y.ArgExpression expr) subst prog = do
   (TeBytes bts) <- _dataize [Y.ArgExpression expr] subst prog
-  pure (btsToUnescapedStr bts)
-argToStrBytes arg _ _ = throwIO (userError (printf "Can't extract bytes from given argument: %s" (prettyExtraArg arg)))
+  pure bts
+argToBytes arg _ _ = throwIO (userError (printf "Can't extract bytes from given argument: %s" (prettyExtraArg arg)))
 
-_contextualize :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+argToString :: Y.ExtraArgument -> Subst -> Program -> IO String
+argToString arg subst prog = argToBytes arg subst prog <&> btsToUnescapedStr
+
+argToNumber :: Y.ExtraArgument -> Subst -> Program -> IO Double
+argToNumber arg subst prog = argToBytes arg subst prog <&> either toDouble id . btsToNum
+
+_contextualize :: BuildTermMethod
 _contextualize [Y.ArgExpression expr, Y.ArgExpression context] subst prog = do
   (expr', _) <- buildExpressionThrows expr subst
   (context', _) <- buildExpressionThrows context subst
   pure (TeExpression (contextualize expr' context' prog))
 _contextualize _ _ _ = throwIO (userError "Function contextualize() requires exactly 2 arguments as expression")
 
-_scope :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_scope :: BuildTermMethod
 _scope [Y.ArgExpression expr] subst _ = do
   (_, scope) <- buildExpressionThrows expr subst
   pure (TeExpression scope)
 _scope _ _ _ = throwIO (userError "Function scope() requires exactly 1 argument as expression")
 
-_randomTau :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_randomTau :: BuildTermMethod
 _randomTau args subst _ = do
   attrs <- argsToAttrs args
   tau <- randomTau attrs
@@ -79,23 +87,14 @@ _randomTau args subst _ = do
       Y.ArgBinding bd -> do
         bds <- buildBindingThrows bd subst
         rest' <- argsToAttrs rest
-        pure (attrsFromBindings bds ++ rest')
+        pure (map show (attributesFromBindings bds) ++ rest')
       Y.ArgBytes _ -> throwIO (userError "Bytes can't be argument of random-tau() function")
-    attrsFromBindings :: [Binding] -> [String]
-    attrsFromBindings [] = []
-    attrsFromBindings (bd : bds) =
-      let attr = case bd of
-            BiTau attr _ -> attr
-            BiDelta _ -> AtDelta
-            BiLambda _ -> AtLambda
-            BiVoid attr -> attr
-       in prettyAttribute attr : attrsFromBindings bds
     randomTau :: [String] -> IO String
     randomTau attrs = do
       tau <- randomString "a🌵%d"
       if tau `elem` attrs then randomTau attrs else pure tau
 
-_dataize :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_dataize :: BuildTermMethod
 _dataize [Y.ArgBytes bytes] subst _ = do
   bts <- buildBytesThrows bytes subst
   pure (TeBytes bts)
@@ -106,18 +105,18 @@ _dataize [Y.ArgExpression expr] subst _ = do
     _ -> throwIO (userError "Only data objects and bytes are supported by 'dataize' function now")
 _dataize _ _ _ = throwIO (userError "Function dataize() requires exactly 1 argument as expression or bytes")
 
-_concat :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_concat :: BuildTermMethod
 _concat args subst prog = do
-  args' <- traverse (\arg -> argToStrBytes arg subst prog) args
+  args' <- traverse (\arg -> argToString arg subst prog) args
   pure (TeExpression (DataString (strToBts (concat args'))))
 
-_sed :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_sed :: BuildTermMethod
 _sed args subst prog = do
   when (length args < 2) (throwIO (userError "Function sed() requires at least two arguments"))
   args' <-
     traverse
       ( \arg -> do
-          bts <- argToStrBytes arg subst prog
+          bts <- argToString arg subst prog
           pure (B.pack bts)
       )
       args
@@ -156,27 +155,27 @@ _sed args subst prog = do
         | h == '/' -> (acc, rest)
         | otherwise -> nextUntilSlash rest (B.snoc acc h) escape
 
-_randomString :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_randomString :: BuildTermMethod
 _randomString [arg] subst prog = do
-  pat <- argToStrBytes arg subst prog
+  pat <- argToString arg subst prog
   str <- randomString pat
   pure (TeExpression (DataString (strToBts str)))
 _randomString _ _ _ = throwIO (userError "Function random-string() requires exactly 1 dataizable argument")
 
-_size :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_size :: BuildTermMethod
 _size [Y.ArgBinding (BiMeta meta)] subst _ = do
   bds <- buildBindingThrows (BiMeta meta) subst
   pure (TeExpression (DataNumber (numToBts (fromIntegral (length bds)))))
 _size _ _ _ = throwIO (userError "Function size() requires exactly 1 meta binding")
 
-_tau :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_tau :: BuildTermMethod
 _tau [Y.ArgExpression expr] subst prog = do
   TeBytes bts <- _dataize [Y.ArgExpression expr] subst prog
   attr <- parseAttributeThrows (btsToUnescapedStr bts)
   pure (TeAttribute attr)
 _tau _ _ _ = throwIO (userError "Function tau() requires exactly 1 argument as expression")
 
-_string :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_string :: BuildTermMethod
 _string [Y.ArgExpression expr] subst _ = do
   (expr', _) <- buildExpressionThrows expr subst
   str <- case expr' of
@@ -196,7 +195,7 @@ _string [Y.ArgAttribute attr] subst _ = do
   pure (TeExpression (DataString (strToBts (prettyAttribute attr'))))
 _string _ _ _ = throwIO (userError "Function string() requires exactly 1 argument as attribute or data expression (Φ̇.number or Φ̇.string)")
 
-_number :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_number :: BuildTermMethod
 _number [Y.ArgExpression expr] subst _ = do
   (expr', _) <- buildExpressionThrows expr subst
   case expr' of
@@ -206,5 +205,42 @@ _number [Y.ArgExpression expr] subst _ = do
     _ -> throwIO (userError (printf "Function number() expects expression to be 'Φ̇.string', but got:\n%s" (prettyExpression' expr')))
 _number _ _ _ = throwIO (userError "Function number() requires exactly 1 argument as 'Φ̇.string'")
 
-_unsupported :: String -> [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_sum :: BuildTermMethod
+_sum args subst prog = do
+  nums <- traverse (\arg -> argToNumber arg subst prog) args
+  pure (TeExpression (DataNumber (numToBts (sum nums))))
+
+_join :: BuildTermMethod
+_join [] _ _ = pure (TeBindings [])
+_join args subst _ = do
+  bds <- buildBindings args
+  pure (TeBindings (join' bds Set.empty))
+  where
+    buildBindings :: [Y.ExtraArgument] -> IO [Binding]
+    buildBindings [] = pure []
+    buildBindings (Y.ArgBinding bd : args') = do
+      bds <- buildBindingThrows bd subst
+      next <- buildBindings args'
+      pure (bds ++ next)
+    buildBindings _ = throwIO (userError "Function 'join' can work with bindings only")
+    join' :: [Binding] -> Set.Set Attribute -> [Binding]
+    join' [] _ = []
+    join' (bd : bds) attrs =
+      let [attr] = attributesFromBindings [bd]
+       in if Set.member attr attrs
+            then
+              if attr == AtRho || attr == AtDelta || attr == AtLambda
+                then join' bds attrs
+                else
+                  let new = case bd of
+                        BiTau attr expr -> BiTau (updated attr attrs) expr
+                        BiVoid attr -> BiVoid (updated attr attrs)
+                   in new : join' bds attrs
+            else bd : join' bds (Set.insert attr attrs)
+    updated :: Attribute -> Set.Set Attribute -> Attribute
+    updated attr attrs =
+      let (TeAttribute attr') = unsafePerformIO (_randomTau (map Y.ArgAttribute (Set.toList attrs)) subst (Program ExGlobal))
+       in attr'
+
+_unsupported :: BuildTermFunc
 _unsupported func _ _ _ = throwIO (userError (printf "Function %s() is not supported or does not exist" func))
\ No newline at end of file
