@@ -22,16 +22,14 @@ where
 import Ast
 import Control.Exception (Exception, throwIO)
 import Control.Monad (guard)
-import Data.Char (isAsciiLower, isDigit, isLower)
+import Data.Char (isAsciiLower, isDigit)
 import Data.Scientific (toRealFloat)
-import Data.Sequence (mapWithIndex)
-import Data.Text.Internal.Fusion.Size (lowerBound)
 import Data.Void
-import GHC.Char (chr)
+import GHC.Char
 import Misc
-import Numeric (readHex)
+import Numeric
 import Text.Megaparsec
-import Text.Megaparsec.Char (alphaNumChar, char, digitChar, hexDigitChar, letterChar, lowerChar, space1, string, upperChar)
+import Text.Megaparsec.Char
 import qualified Text.Megaparsec.Char.Lexer as L
 import Text.Printf (printf)
 
@@ -82,7 +80,7 @@ escapedChar = do
     'f' -> return '\f'
     'u' -> unicodeEscape
     'x' -> hexEscape
-    _ -> fail $ "Unknown escape: \\" ++ [c]
+    _ -> fail ("Unknown escape: \\" ++ [c])
   where
     unicodeEscape :: Parser Char
     unicodeEscape = do
@@ -115,7 +113,7 @@ escapedChar = do
       digits <- count 2 hexDigitChar
       case readHex digits of
         [(n, "")] -> return (chr n)
-        _ -> fail $ "Invalid hex escape: \\x" ++ digits
+        _ -> fail ("Invalid hex escape: \\x" ++ digits)
 
 function :: Parser String
 function = lexeme $ do
@@ -210,13 +208,17 @@ number :: Parser Expression
 number = do
   sign <- optional (choice [char '-', char '+'])
   unsigned <- lexeme L.scientific
-  let num =
-        toRealFloat
-          ( case sign of
-              Just '-' -> negate unsigned
-              _ -> unsigned
-          )
-  return (DataNumber (numToBts num))
+  return
+    ( DataNumber
+        ( numToBts
+            ( toRealFloat
+                ( case sign of
+                    Just '-' -> negate unsigned
+                    _ -> unsigned
+                )
+            )
+        )
+    )
 
 tauBinding :: Parser Attribute -> Parser Binding
 tauBinding attr = do
@@ -227,12 +229,21 @@ tauBinding attr = do
         BiTau attr' <$> expression,
       do
         _ <- symbol "("
-        voids <- map BiVoid <$> void' `sepBy` symbol ","
-        _ <- symbol ")"
+        voids <-
+          choice
+            [ rb >> return [],
+              do
+                voids' <- map BiVoid <$> void' `sepBy1` symbol ","
+                rb >> return voids'
+            ]
         _ <- arrow
-        ExFormation bs <- formation
-        return (BiTau attr' (ExFormation (withVoidRho (voids ++ bs))))
+        bs <- formationBindings
+        bds <- validatedBindings (voids ++ bs)
+        return (BiTau attr' (ExFormation (withVoidRho bds)))
     ]
+  where
+    rb :: Parser String
+    rb = symbol ")"
 
 metaBinding :: Parser Binding
 metaBinding = BiMeta <$> meta' 'B' "𝐵"
@@ -312,13 +323,24 @@ fullAttribute =
     ]
     <?> "full attribute"
 
+validatedBindings :: [Binding] -> Parser [Binding]
+validatedBindings bds = case uniqueBindings bds of
+  Left msg -> fail msg
+  Right bds' -> return bds'
+
 -- formation
-formation :: Parser Expression
-formation = do
+formationBindings :: Parser [Binding]
+formationBindings = do
   _ <- choice [symbol "[[", symbol "⟦"]
-  bs <- binding `sepBy` symbol ","
-  _ <- choice [symbol "]]", symbol "⟧"]
-  return (ExFormation bs)
+  choice
+    [ rsb >> return [],
+      do
+        bs <- binding `sepBy1` symbol ","
+        rsb >> return bs
+    ]
+  where
+    rsb :: Parser String
+    rsb = choice [symbol "]]", symbol "⟧"]
 
 -- head part of expression
 -- 1. formation
@@ -331,7 +353,7 @@ exHead :: Parser Expression
 exHead =
   choice
     [ do
-        ExFormation bs <- formation
+        bs <- formationBindings >>= validatedBindings
         return (ExFormation (withVoidRho bs)),
       do
         _ <- choice [symbol "$", symbol "ξ"]
@@ -351,7 +373,7 @@ exHead =
         str <- manyTill (choice [escapedChar, noneOf ['\\', '"']]) (char '"')
         return (DataString (strToBts str)),
       try (ExMeta <$> meta' 'e' "𝑒"),
-      ExDispatch ExThis <$> fullAttribute
+      ExDispatch ExThis <$> attribute
     ]
     <?> "expression head"
 
@@ -370,7 +392,7 @@ exTail expr =
           choice
             [ do
                 _ <- symbol "."
-                ExDispatch expr <$> fullAttribute,
+                ExDispatch expr <$> attribute,
               do
                 guard
                   ( case expr of
