@@ -6,7 +6,17 @@
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
 
-module XMIR (programToXMIR, printXMIR, toName, parseXMIR, parseXMIRThrows, xmirToPhi, XmirContext (XmirContext)) where
+module XMIR
+  ( programToXMIR,
+    printXMIR,
+    toName,
+    parseXMIR,
+    parseXMIRThrows,
+    xmirToPhi,
+    defaultXmirContext,
+    XmirContext (XmirContext),
+  )
+where
 
 import Ast
 import Control.Exception (Exception (displayException), SomeException, throwIO)
@@ -26,11 +36,11 @@ import Data.Time
 import Data.Time.Clock.POSIX (utcTimeToPOSIXSeconds)
 import Data.Time.Format (defaultTimeLocale, formatTime)
 import Data.Version (showVersion)
+import Debug.Trace
 import Misc
 import Paths_phino (version)
-import Pretty (PrintMode, prettyAttribute, prettyBinding, prettyBytes, prettyExpression, prettyProgram')
+import Pretty (PrintMode (SWEET), prettyAttribute, prettyBinding, prettyBytes, prettyExpression, prettyProgram)
 import Text.Printf (printf)
-import Text.Read (readMaybe)
 import qualified Text.Read as TR
 import Text.XML
 import qualified Text.XML.Cursor as C
@@ -38,22 +48,23 @@ import qualified Text.XML.Cursor as C
 data XmirContext = XmirContext
   { omitListing :: Bool,
     omitComments :: Bool,
-    printMode :: PrintMode
+    listing :: String
   }
 
--- @todo #116:30min Refactor XMIR module. This module became so big and hard to read.
---  Now it's responsible for 3 different operations: 1) converting Phi AST to XML Document Ast,
---  2) printing XML Document, 3) parsing XMIR to Phi AST. I think we should separate the logic
---  in order to keep modules as little as possible.
+defaultXmirContext :: XmirContext
+defaultXmirContext = XmirContext True True ""
+
 data XMIRException
-  = UnsupportedExpression {expr :: Expression}
+  = UnsupportedProgram {prog :: Program}
+  | UnsupportedExpression {expr :: Expression}
   | UnsupportedBinding {binding :: Binding}
   | CouldNotParseXMIR {message :: String}
   | InvalidXMIRFormat {message :: String, cursor :: C.Cursor}
   deriving (Exception)
 
 instance Show XMIRException where
-  show UnsupportedExpression {..} = printf "XMIR does not support such expression: %s" (prettyExpression expr)
+  show UnsupportedProgram {..} = printf "XMIR does not support such program:\n%s" (prettyProgram prog)
+  show UnsupportedExpression {..} = printf "XMIR does not support such expression:\n%s" (prettyExpression expr)
   show UnsupportedBinding {..} = printf "XMIR does not support such bindings: %s" (prettyBinding binding)
   show CouldNotParseXMIR {..} = printf "Couldn't parse given XMIR, cause: %s" message
   show InvalidXMIRFormat {..} =
@@ -77,24 +88,15 @@ element name attrs children = do
 object :: [(String, String)] -> [Node] -> Node
 object attrs children = NodeElement (element "o" attrs children)
 
--- @todo #278:30min Remove xmirAttributure and replace to prettyAttribute. Right now XMIR does not
---  support "ρ" and "φ" in @base and @name attributes. When it's done, we should also generate 
---  such valid XMIR. To achieve that we should get rid of xmirAttribute function and use prettyAttribute
---  instead. Also we should use "@" in formationBinding. Don't forget to remove the puzzle. 
-xmirAttribute :: Attribute -> String
-xmirAttribute AtPhi = "@"
-xmirAttribute AtRho = "^"
-xmirAttribute other = prettyAttribute other
-
 expression :: Expression -> XmirContext -> IO (String, [Node])
-expression ExThis _ = pure ("$", [])
-expression ExGlobal _ = pure ("Q", [])
+expression ExThis _ = pure (prettyExpression ExThis, [])
+expression ExGlobal _ = pure (prettyExpression ExGlobal, [])
 expression (ExFormation bds) ctx = do
   nested <- nestedBindings bds ctx
   pure ("", nested)
 expression (ExDispatch expr attr) ctx = do
   (base, children) <- expression expr ctx
-  let attr' = xmirAttribute attr
+  let attr' = prettyAttribute attr
   if null base
     then pure ('.' : attr', [object [] children])
     else
@@ -104,10 +106,10 @@ expression (ExDispatch expr attr) ctx = do
 expression (DataNumber bytes) XmirContext {..} =
   let bts =
         object
-          [("as", prettyAttribute (AtAlpha 0)), ("base", "Q.org.eolang.bytes")]
+          [("as", prettyAttribute (AtAlpha 0)), ("base", "Φ.org.eolang.bytes")]
           [object [] [NodeContent (T.pack (prettyBytes bytes))]]
    in pure
-        ( "Q.org.eolang.number",
+        ( "Φ.org.eolang.number",
           if omitComments
             then [bts]
             else
@@ -118,10 +120,10 @@ expression (DataNumber bytes) XmirContext {..} =
 expression (DataString bytes) XmirContext {..} =
   let bts =
         object
-          [("as", prettyAttribute (AtAlpha 0)), ("base", "Q.org.eolang.bytes")]
+          [("as", prettyAttribute (AtAlpha 0)), ("base", "Φ.org.eolang.bytes")]
           [object [] [NodeContent (T.pack (prettyBytes bytes))]]
    in pure
-        ( "Q.org.eolang.string",
+        ( "Φ.org.eolang.string",
           if omitComments
             then [bts]
             else
@@ -132,7 +134,7 @@ expression (DataString bytes) XmirContext {..} =
 expression (ExApplication expr (BiTau attr texpr)) ctx = do
   (base, children) <- expression expr ctx
   (base', children') <- expression texpr ctx
-  let as = xmirAttribute attr
+  let as = prettyAttribute attr
       attrs =
         if null base'
           then [("as", as)]
@@ -141,9 +143,6 @@ expression (ExApplication expr (BiTau attr texpr)) ctx = do
 expression (ExApplication (ExFormation bds) tau) _ = throwIO (UnsupportedExpression (ExApplication (ExFormation bds) tau))
 expression expr _ = throwIO (UnsupportedExpression expr)
 
-nestedBindings :: [Binding] -> XmirContext -> IO [Node]
-nestedBindings bds ctx = catMaybes <$> mapM (`formationBinding` ctx) bds
-
 formationBinding :: Binding -> XmirContext -> IO (Maybe Node)
 formationBinding (BiTau (AtLabel label) (ExFormation bds)) ctx = do
   inners <- nestedBindings bds ctx
@@ -153,97 +152,111 @@ formationBinding (BiTau (AtLabel label) expr) ctx = do
   pure (Just (object [("name", label), ("base", base)] children))
 formationBinding (BiTau AtPhi expr) ctx = do
   (base, children) <- expression expr ctx
-  pure (Just (object [("name", "@"), ("base", base)] children))
+  pure (Just (object [("name", show AtPhi), ("base", base)] children))
 formationBinding (BiTau AtRho _) _ = pure Nothing
 formationBinding (BiDelta bytes) _ = pure (Just (NodeContent (T.pack (prettyBytes bytes))))
-formationBinding (BiLambda func) _ = pure (Just (object [("name", "λ")] []))
+formationBinding (BiLambda func) _ = pure (Just (object [("name", show AtLambda)] []))
 formationBinding (BiVoid AtRho) _ = pure Nothing
-formationBinding (BiVoid AtPhi) _ = pure (Just (object [("name", "@"), ("base", "∅")] []))
+formationBinding (BiVoid AtPhi) _ = pure (Just (object [("name", show AtPhi), ("base", "∅")] []))
 formationBinding (BiVoid (AtLabel label)) _ = pure (Just (object [("name", label), ("base", "∅")] []))
 formationBinding binding _ = throwIO (UnsupportedBinding binding)
 
-rootExpression :: Expression -> XmirContext -> IO Node
-rootExpression (ExFormation [bd, BiVoid AtRho]) ctx = do
-  [bd'] <- nestedBindings [bd] ctx
-  pure bd'
-rootExpression expr _ = throwIO (UnsupportedExpression expr)
-
--- Extract package from given expression
--- The function returns tuple (X, Y), where
--- - X: list of package parts
--- - Y: root object expression
--- @todo #197:30min Make patterns with L> Package softer. Right now we expect L> Package only in the end
---  of the formation bindings list. That's not really correct since this binding may be anywhere. Let's fix it
-getPackage :: Expression -> IO ([String], Expression)
-getPackage (ExFormation [BiTau (AtLabel label) (ExFormation [bd, BiLambda "Package", BiVoid AtRho]), BiVoid AtRho]) = do
-  (pckg, expr') <- getPackage (ExFormation [bd, BiLambda "Package", BiVoid AtRho])
-  pure (label : pckg, expr')
-getPackage (ExFormation [BiTau (AtLabel label) (ExFormation [bd, BiLambda "Package", BiVoid AtRho]), BiLambda "Package", BiVoid AtRho]) = do
-  (pckg, expr') <- getPackage (ExFormation [bd, BiLambda "Package", BiVoid AtRho])
-  pure (label : pckg, expr')
-getPackage (ExFormation [BiTau attr expr, BiLambda "Package", BiVoid AtRho]) = pure ([], ExFormation [BiTau attr expr, BiVoid AtRho])
-getPackage (ExFormation [bd, BiVoid AtRho]) = pure ([], ExFormation [bd, BiVoid AtRho])
-getPackage expr = throwIO (userError (printf "Can't extract package from given expression:\n %s" (prettyExpression expr)))
-
-metasWithPackage :: String -> Node
-metasWithPackage pckg =
-  NodeElement
-    ( element
-        "metas"
-        []
-        [ NodeElement
-            ( element
-                "meta"
-                []
-                [ NodeElement (element "head" [] [NodeContent (T.pack "package")]),
-                  NodeElement (element "tail" [] [NodeContent (T.pack pckg)]),
-                  NodeElement (element "part" [] [NodeContent (T.pack pckg)])
-                ]
-            )
-        ]
-    )
-
-time :: UTCTime -> String
-time now = do
-  let base = formatTime defaultTimeLocale "%Y-%m-%dT%H:%M:%S" now
-      posix = utcTimeToPOSIXSeconds now
-      fractional :: Double
-      fractional = realToFrac posix - fromInteger (floor posix)
-      nanos = floor (fractional * 1_000_000_000) :: Int
-  base ++ "." ++ printf "%09d" nanos ++ "Z"
+nestedBindings :: [Binding] -> XmirContext -> IO [Node]
+nestedBindings bds ctx = catMaybes <$> mapM (`formationBinding` ctx) bds
 
 programToXMIR :: Program -> XmirContext -> IO Document
-programToXMIR (Program expr) ctx = do
-  (pckg, expr') <- getPackage expr
-  root <- rootExpression expr' ctx
-  now <- getCurrentTime
-  let phi = prettyProgram' (Program expr) (printMode ctx)
-      listing =
-        if omitListing ctx
-          then show (length (lines phi)) ++ " lines of phi"
-          else phi
-      listing' = NodeElement (element "listing" [] [NodeContent (T.pack listing)])
-      metas = metasWithPackage (intercalate "." pckg)
-  pure
-    ( Document
-        (Prologue [] Nothing [])
-        ( element
-            "object"
-            [ ("dob", formatTime defaultTimeLocale "%Y-%m-%dT%H:%M:%S" now),
-              ("ms", "0"),
-              ("revision", "1234567"),
-              ("time", time now),
-              ("version", showVersion version),
-              ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance"),
-              ("xsi:noNamespaceSchemaLocation", "https://raw.githubusercontent.com/objectionary/eo/refs/heads/gh-pages/XMIR.xsd")
-            ]
-            ( if null pckg
-                then [listing', root]
-                else [listing', metas, root]
+programToXMIR prog@(Program expr@(ExFormation [BiTau (AtLabel _) arg, BiVoid AtRho])) ctx@XmirContext {..} = case arg of
+  ExFormation _ -> programToXMIR'
+  ExApplication _ _ -> programToXMIR'
+  ExDispatch _ _ -> programToXMIR'
+  ExGlobal -> programToXMIR'
+  _ -> throwIO (UnsupportedProgram prog)
+  where
+    programToXMIR' :: IO Document
+    programToXMIR' = do
+      (pckg, expr') <- getPackage expr
+      root <- rootExpression expr' ctx
+      now <- getCurrentTime
+      let listingContent =
+            if omitListing
+              then show (length (lines listing)) ++ " line(s)"
+              else listing
+          listing' = NodeElement (element "listing" [] [NodeContent (T.pack listingContent)])
+          metas = metasWithPackage (intercalate "." pckg)
+      pure
+        ( Document
+            (Prologue [] Nothing [])
+            ( element
+                "object"
+                [ ("dob", formatTime defaultTimeLocale "%Y-%m-%dT%H:%M:%S" now),
+                  ("ms", "0"),
+                  ("revision", "1234567"),
+                  ("time", time now),
+                  ("version", showVersion version),
+                  ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance"),
+                  ("xsi:noNamespaceSchemaLocation", "https://raw.githubusercontent.com/objectionary/eo/refs/heads/gh-pages/XMIR.xsd")
+                ]
+                ( if null pckg
+                    then [listing', root]
+                    else [listing', metas, root]
+                )
             )
+            []
         )
-        []
-    )
+    -- Extract package from given expression
+    -- The function returns tuple (X, Y), where
+    -- - X: list of package parts
+    -- - Y: root object expression
+    getPackage :: Expression -> IO ([String], Expression)
+    getPackage (ExFormation [BiTau (AtLabel label) (ExFormation [bd, BiLambda "Package", BiVoid AtRho]), BiVoid AtRho]) = do
+      (pckg, expr') <- getPackage (ExFormation [bd, BiLambda "Package", BiVoid AtRho])
+      pure (label : pckg, expr')
+    getPackage (ExFormation [BiTau (AtLabel label) (ExFormation [bd, BiLambda "Package", BiVoid AtRho]), BiLambda "Package", BiVoid AtRho]) = do
+      (pckg, expr') <- getPackage (ExFormation [bd, BiLambda "Package", BiVoid AtRho])
+      pure (label : pckg, expr')
+    getPackage (ExFormation [BiTau attr expr, BiLambda "Package", BiVoid AtRho]) = pure ([], ExFormation [BiTau attr expr, BiVoid AtRho])
+    getPackage (ExFormation [bd, BiVoid AtRho]) = pure ([], ExFormation [bd, BiVoid AtRho])
+    getPackage expr = throwIO (userError (printf "Can't extract package from given expression:\n %s" (prettyExpression expr)))
+    -- Convert root Expression to Node
+    rootExpression :: Expression -> XmirContext -> IO Node
+    rootExpression (ExFormation [bd, BiVoid AtRho]) ctx = do
+      [bd'] <- nestedBindings [bd] ctx
+      pure bd'
+    rootExpression expr _ = throwIO (UnsupportedExpression expr)
+    -- Returns metas Node with package:
+    -- <metas>
+    --   <meta>
+    --     <head>package</head>
+    --     <tail><!-- package here --></tail>
+    --     <part><!-- package here --></part>
+    --   </meta>
+    -- </metas>
+    metasWithPackage :: String -> Node
+    metasWithPackage pckg =
+      NodeElement
+        ( element
+            "metas"
+            []
+            [ NodeElement
+                ( element
+                    "meta"
+                    []
+                    [ NodeElement (element "head" [] [NodeContent (T.pack "package")]),
+                      NodeElement (element "tail" [] [NodeContent (T.pack pckg)]),
+                      NodeElement (element "part" [] [NodeContent (T.pack pckg)])
+                    ]
+                )
+            ]
+        )
+    time :: UTCTime -> String
+    time now = do
+      let base = formatTime defaultTimeLocale "%Y-%m-%dT%H:%M:%S" now
+          posix = utcTimeToPOSIXSeconds now
+          fractional :: Double
+          fractional = realToFrac posix - fromInteger (floor posix)
+          nanos = floor (fractional * 1_000_000_000) :: Int
+      base ++ "." ++ printf "%09d" nanos ++ "Z"
+programToXMIR prog _ = throwIO (UnsupportedProgram prog)
 
 -- Add indentation (2 spaces per level).
 indent :: Int -> TB.Builder
@@ -363,17 +376,17 @@ xmirToFormationBinding cur fqn
   | not (hasAttr "name" cur) = throwIO (InvalidXMIRFormat "Formation children must have @name attribute" cur)
   | not (hasAttr "base" cur) = do
       name <- getAttr "name" cur
-      bds <- mapM (`xmirToFormationBinding` (name : fqn)) (cur C.$/ C.element (toName "o"))
+      bds <- mapM (`xmirToFormationBinding` (name : fqn)) (cur C.$/ C.element (toName "o")) >>= uniqueBindings'
       case name of
         "λ" -> pure (BiLambda (intercalate "_" ("L" : reverse fqn)))
         ('α' : _) -> throwIO (InvalidXMIRFormat "Formation child @name can't start with α" cur)
-        "@" -> pure (BiTau AtPhi (ExFormation (withVoidRho bds)))
+        "φ" -> pure (BiTau AtPhi (ExFormation (withVoidRho bds)))
         _ -> pure (BiTau (AtLabel name) (ExFormation (withVoidRho bds)))
   | otherwise = do
       name <- getAttr "name" cur
       base <- getAttr "base" cur
       attr <- case name of
-        "@" -> pure AtPhi
+        "φ" -> pure AtPhi
         ('α' : _) -> throwIO (InvalidXMIRFormat "Formation child @name can't start with α" cur)
         _ -> pure (AtLabel name)
       case base of
@@ -399,19 +412,19 @@ xmirToExpression cur fqn
                       attr <- toAttr rest cur
                       let disp = ExDispatch expr attr
                       xmirToApplication disp (tail args) fqn
-        "$" ->
+        "ξ" ->
           if null (cur C.$/ C.element (toName "o"))
             then pure ExThis
-            else throwIO (InvalidXMIRFormat "Application of '$' is illegal in XMIR" cur)
-        "Q" ->
+            else throwIO (InvalidXMIRFormat "Application of 'ξ' is illegal in XMIR" cur)
+        "Φ" ->
           if null (cur C.$/ C.element (toName "o"))
             then pure ExGlobal
-            else throwIO (InvalidXMIRFormat "Application of 'Q' is illegal in XMIR" cur)
-        'Q' : '.' : rest -> xmirToExpression' ExGlobal "Q" rest cur fqn
-        '$' : '.' : rest -> xmirToExpression' ExThis "$" rest cur fqn
-        _ -> throwIO (InvalidXMIRFormat "The @base attribute must be either ['∅'|'Q'] or start with ['Q.'|'$.'|'.']" cur)
+            else throwIO (InvalidXMIRFormat "Application of 'Φ' is illegal in XMIR" cur)
+        'Φ' : '.' : rest -> xmirToExpression' ExGlobal "Φ" rest cur fqn
+        'ξ' : '.' : rest -> xmirToExpression' ExThis "ξ" rest cur fqn
+        _ -> throwIO (InvalidXMIRFormat "The @base attribute must be either ['∅'|'Φ'] or start with ['Φ.'|'ξ.'|'.']" cur)
   | otherwise = do
-      bds <- mapM (`xmirToFormationBinding` fqn) (cur C.$/ C.element (toName "o"))
+      bds <- mapM (`xmirToFormationBinding` fqn) (cur C.$/ C.element (toName "o")) >>= uniqueBindings'
       pure (ExFormation (withVoidRho bds))
   where
     xmirToExpression' :: Expression -> String -> String -> C.Cursor -> [String] -> IO Expression
@@ -424,8 +437,7 @@ xmirToExpression cur fqn
               (\acc part -> ExDispatch acc <$> toAttr (T.unpack part) cur)
               start
               (T.splitOn "." (T.pack rest))
-          let args = cur C.$/ C.element (toName "o")
-          xmirToApplication head' args fqn
+          xmirToApplication head' (cur C.$/ C.element (toName "o")) fqn
 
 xmirToApplication :: Expression -> [C.Cursor] -> [String] -> IO Expression
 xmirToApplication = xmirToApplication' 0
@@ -457,7 +469,7 @@ xmirToApplication = xmirToApplication' 0
           as <- getAttr "as" cur
           attr <- toAttr as cur
           case attr of
-            AtRho -> throwIO (InvalidXMIRFormat "The '^' in @as attribute is illegal in XMIR" cur)
+            AtRho -> throwIO (InvalidXMIRFormat "The 'ρ' in @as attribute is illegal in XMIR" cur)
             other -> pure other
       | otherwise = pure (AtAlpha idx)
 
@@ -467,8 +479,8 @@ toAttr attr cur = case attr of
     case TR.readMaybe rest' :: Maybe Integer of
       Just idx -> pure (AtAlpha idx)
       Nothing -> throwIO (InvalidXMIRFormat "The attribute started with 'α' must be followed by integer" cur)
-  "@" -> pure AtPhi
-  "^" -> pure AtRho
+  "φ" -> pure AtPhi
+  "ρ" -> pure AtRho
   _
     | head attr `notElem` ['a' .. 'z'] -> throwIO (InvalidXMIRFormat (printf "The attribute '%s' must start with ['a'..'z']" attr) cur)
     | '.' `elem` attr -> throwIO (InvalidXMIRFormat "Attribute can't contain dots" cur)
