@@ -99,15 +99,12 @@ spec = do
                 )
             )
         ),
-        ( "[[x -> y.z, a -> ~1, w -> ^, u -> @, p -> !a, q -> !e]]",
+        ( "[[x -> y.z, w -> ^, u -> @, p -> !a, q -> !e]]",
           Just
             ( ExFormation
                 [ BiTau
                     (AtLabel "x")
                     (ExDispatch (ExDispatch ExThis (AtLabel "y")) (AtLabel "z")),
-                  BiTau
-                    (AtLabel "a")
-                    (ExDispatch ExThis (AtAlpha 1)),
                   BiTau
                     (AtLabel "w")
                     (ExDispatch ExThis AtRho),
@@ -124,21 +121,18 @@ spec = do
                 ]
             )
         ),
-        ( "Q.x(~1, y, [[]].z, Q.y(^,@))",
+        ( "Q.x(y, [[]].z, Q.y(^,@))",
           Just
             ( ExApplication
                 ( ExApplication
                     ( ExApplication
-                        ( ExApplication
-                            (ExDispatch ExGlobal (AtLabel "x"))
-                            (BiTau (AtAlpha 0) (ExDispatch ExThis (AtAlpha 1)))
-                        )
-                        (BiTau (AtAlpha 1) (ExDispatch ExThis (AtLabel "y")))
+                        (ExDispatch ExGlobal (AtLabel "x"))
+                        (BiTau (AtAlpha 0) (ExDispatch ExThis (AtLabel "y")))
                     )
-                    (BiTau (AtAlpha 2) (ExDispatch (ExFormation [BiVoid AtRho]) (AtLabel "z")))
+                    (BiTau (AtAlpha 1) (ExDispatch (ExFormation [BiVoid AtRho]) (AtLabel "z")))
                 )
                 ( BiTau
-                    (AtAlpha 3)
+                    (AtAlpha 2)
                     ( ExApplication
                         ( ExApplication
                             (ExDispatch ExGlobal (AtLabel "y"))
@@ -229,17 +223,15 @@ spec = do
         "Q.x(y() -> [[]])",
         "Q.x(y(q) -> [[w -> !e]])",
         "Q.x(~1(^,@) -> [[]])",
-        "Q.x.~1.^.@.!a0",
+        "Q.x.^.@.!a0",
         "[[x -> y.z]]",
-        "[[x -> ~1]]",
         "[[x -> ^, y -> @, z -> !a]]",
         "Q.x(a.b.c, Q.a(b), [[]])",
-        "Q.x(~1, y, [[]].z, Q.y(^,@))",
+        "Q.x(y, [[]].z, Q.y(^,@))",
         "[[x -> 5.plus(5), y -> \"hello\", z -> 42.5]]",
-        "[[w -> x(~1)]]",
         "[[\n  x -> \"Hi\",\n  y -> 42\n]]",
         "[[x -> -42, y -> +34]]",
-        "⟦x ↦ Φ.org.eolang(z ↦ ξ.f, x ↦ α0, φ ↦ ρ, t ↦ φ, first ↦ ⟦ λ ⤍ Function_name, Δ ⤍ 42- ⟧)⟧",
+        "⟦x ↦ Φ.org.eolang(z ↦ ξ.f, φ ↦ ρ, t ↦ φ, first ↦ ⟦ λ ⤍ Function_name, Δ ⤍ 42- ⟧)⟧",
         "[[x -> 1.00e+3, y -> 2.32e-4]]",
         "[[ x -> \"\\u0001\\u0001\"]]",
         "[[ x -> \"\\uD835\\uDF11\"]]",
@@ -269,11 +261,15 @@ spec = do
             "[[z(w) -> Q.x]]",
             "Q.x(y(~1) -> [[]])",
             "Q.x(1, 2, !B)",
+            "Q.x.~0",
             "Q.x(~1 -> Q.y, x -> 5, !B1)",
             "Q.x(𝐵1, 𝜏0 -> $, x -> 𝑒)",
             "[[ x -> \"\\uD800\"]]",
             "[[ x -> \"\\uDFFF\"]]",
-            "[[ x -> \"\\uD835\\u0041\"]]"
+            "[[ x -> \"\\uD835\\u0041\"]]",
+            "[[ x -> 1, x -> 2 ]]",
+            "⟦ k ↦ ⟦ λ ⤍ Foo, λ ⤍ Bar ⟧ ⟧",
+            "⟦ k ↦ ⟦ Δ ⤍ 42-, Δ ⤍ 55- ⟧ ⟧"
           ]
       )
 
@@ -285,3 +281,12 @@ spec = do
           content <- runIO (readFile pack)
           it (takeBaseName pack) (parseProgram content `shouldSatisfy` isRight)
       )
+
+  describe "process typo packs" $ do
+    packs <- runIO (allPathsIn "test-resources/phi-typos-packs")
+    forM_
+      packs
+      ( \pack -> do
+          content <- runIO (readFile pack)
+          it (takeBaseName pack) (parseProgram content `shouldSatisfy` isLeft)
+      )
